/****************************************************************************************
*
* Control a Runcam Swift Micro 3 FPV-camera over the serial line from an Arduino board
*
* Tested with an Mega 2560. Other Arduinos should work too. Be careful to use correct 
* serial line pins and logic voltage level.
*
* Most Arduinos (including the 2560) use 5V. The Cam has a 3.3V serial line. So you
* should use resistors to connect the serial pins:
*
*                         ___ 680 Ohm ___ 1k Ohm ___ Arduino GND
*                        |
* Arduino Pin 0 (RX) --- |
*                        |___ Runcam TX
*
*                                    ___ 1k Ohm ___ Arduino GND
*                                   |
* Arduino Pin 1 (TX) --- 680 Ohm ---|
*                                   |___ Runcam RX
*
* To "press" a control button connect the according GPIO-pin to GND. Disconnecting the
* pin will "release" the key. You can use cable bridges for this, but two buttons and
* a mini-joystick will be more comfortable.
*
* How to use the controls:
*          - press connect button once
*               - cam goes to remote mode
*          - press set button once
*               - cam goes to setup menu
*          - OR hold the up button for a while
*               - cam goes to OSD config
*          - change settings with directional and set buttons
*          - exit menu
*          - press connect button once
*               - cam leaves remote mode
*
* You use this program and the explanations at your own risk!!!
*
* This is for personal use only. Commercial use is prohibited.
*
* Commercial users may preconfigure items they sale with this tool and give the unchanged
* source to their customers (sent with the product or as download from their homepage)
* as long as they don't take a charge for it.
*
* If this tool has helped you, you have found an error/problem or you want to use it in a way,
* that is currently not allowed , I will be happy to hear from you.
*
* copyright
*      Thomas Goebhardt
*      pizzamampf@pizzamampf.de
*      http://www.pizzamampf.de
*
****************************************************************************************/

//Parameter for serial port
#define SERIALBAUDRATE          115200
#define SERIALTIMEOUT           1
#define SERIALSENDDELAY         100

//GPIO pins
#define PIN_RX                  0 //Serial port pins are fixed by hardwaredesign, connect to Runcam TX (brown)
#define PIN_TX                  1 //Serial port pins are fixed by hardwaredesign, connect to Runcam RX (green)
#define PIN_SET                 2 //Control pins must be in ascending order without gaps
#define PIN_LEFT                3 //Control pins must be in ascending order without gaps
#define PIN_RIGHT               4 //Control pins must be in ascending order without gaps
#define PIN_UP                  5 //Control pins must be in ascending order without gaps
#define PIN_DOWN                6 //Control pins must be in ascending order without gaps
#define PIN_CONNECT_DISCONNECT  7 //Control pins must be in ascending order without gaps

//Runcam coammnd and parameter values
#define RUNCAM_HEADER                    0xCC
#define RUNCAM_COMMAND_PRESS             0x02
#define RUNCAM_COMMAND_RELEASE           0x03
#define RUNCAM_COMMAND_CONNECT           0x04
#define RUNCAM_PARAMETER_SET             0x01
#define RUNCAM_PARAMETER_LEFT            0x02
#define RUNCAM_PARAMETER_RIGHT           0x03
#define RUNCAM_PARAMETER_UP              0x04
#define RUNCAM_PARAMETER_DOWN            0x05
#define RUNCAM_PARAMETER_CONNECT         0x01
#define RUNCAM_PARAMETER_DISCONNECT      0x02

//Variables
int button_states[]={0,0,0,0,0,0}; //tracks button states
int connected_to_cam=0;            //tracks connection status

boolean evaluate_PIN(int pin)
{
  //checks pin and returns true if pressed
  //uses button_states to only submit once if key is not released since last check
  if(!digitalRead(pin) && button_states[pin-PIN_SET]==0)
  {
    //button is pressed
    button_states[pin-PIN_SET]=1;
    return(true);
  }
  else if(digitalRead(pin) && button_states[pin-PIN_SET]==1)
  {
    //button is released
    button_states[pin-PIN_SET]=0;
    release_key();
  }
  
  return(false);
}

void toggle_cam_connection()
{
  //toggle connection to cam
  if(connected_to_cam==0)
  {
    //connect to cam
    Serial.write(RUNCAM_HEADER);
    Serial.write(RUNCAM_COMMAND_CONNECT);
    Serial.write(RUNCAM_PARAMETER_CONNECT);
    Serial.write(0xC0); //crc8_dvb_s2
    Serial.flush();
    delay(SERIALSENDDELAY);
    connected_to_cam=1;
  }
  else
  {
    //disconnect from cam
    Serial.write(RUNCAM_HEADER);
    Serial.write(RUNCAM_COMMAND_CONNECT);
    Serial.write(RUNCAM_PARAMETER_DISCONNECT);
    Serial.write(0x6A); //crc8_dvb_s2
    Serial.flush();
    delay(SERIALSENDDELAY);
    connected_to_cam=0;
  }
}

void release_key()
{
  //release pressed keys
  Serial.write(RUNCAM_HEADER);
  Serial.write(RUNCAM_COMMAND_RELEASE);
  Serial.write(0xCA); //crc8_dvb_s2
  Serial.flush();
  delay(SERIALSENDDELAY);
}

void press_key(unsigned char key,unsigned char crc)
{
  //press the key, correct crc-value must be submitted by caller
  Serial.write(RUNCAM_HEADER);
  Serial.write(RUNCAM_COMMAND_PRESS);
  Serial.write(key);
  Serial.write(crc); //crc8_dvb_s2
  Serial.flush();
  delay(SERIALSENDDELAY);
}

void setup()
{
  //initialize serial port
  Serial.begin(SERIALBAUDRATE);
  Serial.setTimeout(SERIALTIMEOUT);
  
  //initialize GPIO pins
  pinMode(PIN_SET,INPUT_PULLUP);
  pinMode(PIN_LEFT,INPUT_PULLUP);
  pinMode(PIN_RIGHT,INPUT_PULLUP);
  pinMode(PIN_UP,INPUT_PULLUP);
  pinMode(PIN_DOWN,INPUT_PULLUP);
  pinMode(PIN_CONNECT_DISCONNECT,INPUT_PULLUP);
}

void loop()
{
  //toggle cam connection
  if(evaluate_PIN(PIN_CONNECT_DISCONNECT)) toggle_cam_connection();

  //set
  if(evaluate_PIN(PIN_SET)) press_key(RUNCAM_PARAMETER_SET,0xFA);

  //left
  if(evaluate_PIN(PIN_LEFT)) press_key(RUNCAM_PARAMETER_LEFT,0x50);

  //right
  if(evaluate_PIN(PIN_RIGHT)) press_key(RUNCAM_PARAMETER_RIGHT,0x85);

  //up
  if(evaluate_PIN(PIN_UP)) press_key(RUNCAM_PARAMETER_UP,0xD1);

  //down
  if(evaluate_PIN(PIN_DOWN)) press_key(RUNCAM_PARAMETER_DOWN,0x04);

  delay(10);
}

